# Auto generated Python source file from Ada sources
# Call 'make' in 'src/vhdl' to regenerate:
#
from pyTooling.Decorators import export

from pyGHDL.libghdl import libghdl


@export
def Get_Start_Location(obj):
    return libghdl.vhdl__elocations__get_start_location(obj)


@export
def Set_Start_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_start_location(obj, value)


@export
def Get_Right_Paren_Location(obj):
    return libghdl.vhdl__elocations__get_right_paren_location(obj)


@export
def Set_Right_Paren_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_right_paren_location(obj, value)


@export
def Get_End_Location(obj):
    return libghdl.vhdl__elocations__get_end_location(obj)


@export
def Set_End_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_end_location(obj, value)


@export
def Get_Is_Location(obj):
    return libghdl.vhdl__elocations__get_is_location(obj)


@export
def Set_Is_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_is_location(obj, value)


@export
def Get_Begin_Location(obj):
    return libghdl.vhdl__elocations__get_begin_location(obj)


@export
def Set_Begin_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_begin_location(obj, value)


@export
def Get_Then_Location(obj):
    return libghdl.vhdl__elocations__get_then_location(obj)


@export
def Set_Then_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_then_location(obj, value)


@export
def Get_Use_Location(obj):
    return libghdl.vhdl__elocations__get_use_location(obj)


@export
def Set_Use_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_use_location(obj, value)


@export
def Get_Loop_Location(obj):
    return libghdl.vhdl__elocations__get_loop_location(obj)


@export
def Set_Loop_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_loop_location(obj, value)


@export
def Get_Generate_Location(obj):
    return libghdl.vhdl__elocations__get_generate_location(obj)


@export
def Set_Generate_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_generate_location(obj, value)


@export
def Get_Generic_Location(obj):
    return libghdl.vhdl__elocations__get_generic_location(obj)


@export
def Set_Generic_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_generic_location(obj, value)


@export
def Get_Port_Location(obj):
    return libghdl.vhdl__elocations__get_port_location(obj)


@export
def Set_Port_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_port_location(obj, value)


@export
def Get_Generic_Map_Location(obj):
    return libghdl.vhdl__elocations__get_generic_map_location(obj)


@export
def Set_Generic_Map_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_generic_map_location(obj, value)


@export
def Get_Port_Map_Location(obj):
    return libghdl.vhdl__elocations__get_port_map_location(obj)


@export
def Set_Port_Map_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_port_map_location(obj, value)


@export
def Get_Arrow_Location(obj):
    return libghdl.vhdl__elocations__get_arrow_location(obj)


@export
def Set_Arrow_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_arrow_location(obj, value)


@export
def Get_Colon_Location(obj):
    return libghdl.vhdl__elocations__get_colon_location(obj)


@export
def Set_Colon_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_colon_location(obj, value)


@export
def Get_Assign_Location(obj):
    return libghdl.vhdl__elocations__get_assign_location(obj)


@export
def Set_Assign_Location(obj, value) -> None:
    libghdl.vhdl__elocations__set_assign_location(obj, value)
